@@ -6,7 +6,7 @@ from collections import ChainMap
 from contextlib import AsyncExitStack, AbstractAsyncContextManager, asynccontextmanager
 from json import JSONDecodeError
 from types import FunctionType, CoroutineType
-from typing import List, Union, Any, Callable, Type, Optional, Dict, Sequence, Awaitable
+from typing import List, Union, Any, Callable, Type, Optional, Dict, Sequence, Awaitable, Tuple
 
 # noinspection PyProtectedMember
 from pydantic import DictError
@@ -493,18 +493,26 @@ class JsonRpcContext:
         http_request: Request,
         background_tasks: BackgroundTasks,
         http_response: Response,
+        json_rpc_request_class: Type[JsonRpcRequest] = JsonRpcRequest
     ):
         self.entrypoint: Entrypoint = entrypoint
         self.raw_request: Any = raw_request
         self.http_request: Request = http_request
         self.background_tasks: BackgroundTasks = background_tasks
         self.http_response: Response = http_response
+        self.request_class: Type[JsonRpcRequest] = json_rpc_request_class
         self._raw_response: Optional[dict] = None
         self.exception: Optional[Exception] = None
+        self.is_unhandled_exception: bool = False
         self.exit_stack: Optional[AsyncExitStack] = None
         self.jsonrpc_context_token: Optional[contextvars.Token] = None
 
-    def on_raw_response(self, raw_response: dict, exception: Optional[Exception] = None):
+    def on_raw_response(
+        self,
+        raw_response: dict,
+        exception: Optional[Exception] = None,
+        is_unhandled_exception: bool = False,
+    ):
         raw_response.pop('id', None)
         if isinstance(self.raw_request, dict) and 'id' in self.raw_request:
             raw_response['id'] = self.raw_request.get('id')
@@ -512,6 +520,7 @@ class JsonRpcContext:
             raw_response['id'] = None
         self._raw_response = raw_response
         self.exception = exception
+        self.is_unhandled_exception = is_unhandled_exception
 
     @property
     def raw_response(self) -> dict:
@@ -524,7 +533,7 @@ class JsonRpcContext:
     @cached_property
     def request(self) -> JsonRpcRequest:
         try:
-            return JsonRpcRequest.validate(self.raw_request)
+            return self.request_class.validate(self.raw_request)
         except DictError:
             raise InvalidRequest(data={'errors': [{
                 'loc': (),
@@ -553,17 +562,21 @@ class JsonRpcContext:
         except Exception as exc:
             if exc is not self.exception:
                 try:
-                    resp = await self.entrypoint.handle_exception_to_resp(exc)
+                    resp, is_unhandled_exception = await self.entrypoint.handle_exception_to_resp(
+                        exc, log_unhandled_exception=False,
+                    )
                 except Exception as exc:
                     # HTTPException
                     self._raw_response = None
                     self.exception = exc
+                    self.is_unhandled_exception = True
                     raise
-                self.on_raw_response(resp, exc)
+                self.on_raw_response(resp, exc, is_unhandled_exception)
             if reraise:
                 raise
-            else:
-                logger.exception(str(exc), exc_info=exc)
+
+        if self.exception is not None and self.is_unhandled_exception:
+            logger.exception(str(self.exception), exc_info=self.exception)
 
     async def enter_middlewares(self, middlewares: Sequence['JsonRpcMiddleware']):
         for mw in middlewares:
@@ -604,6 +617,7 @@ class MethodRoute(APIRoute):
         errors: Sequence[Type[BaseError]] = None,
         dependencies: Sequence[Depends] = None,
         response_class: Type[Response] = JSONResponse,
+        request_class: Type[JsonRpcRequest] = JsonRpcRequest,
         middlewares: Sequence[JsonRpcMiddleware] = None,
         **kwargs,
     ):
@@ -660,6 +674,7 @@ class MethodRoute(APIRoute):
         self.entrypoint = entrypoint
         self.middlewares = middlewares or []
         self.app = request_response(self.handle_http_request)
+        self.request_class = request_class
 
     async def parse_body(self, http_request) -> Any:
         try:
@@ -683,8 +698,7 @@ class MethodRoute(APIRoute):
         try:
             body = await self.parse_body(http_request)
         except Exception as exc:
-            logger.exception(str(exc), exc_info=exc)
-            resp = await self.entrypoint.handle_exception_to_resp(exc)
+            resp, _ = await self.entrypoint.handle_exception_to_resp(exc)
             response = self.response_class(content=resp, background=background_tasks)
         else:
             try:
@@ -748,6 +762,7 @@ class MethodRoute(APIRoute):
             http_request=http_request,
             background_tasks=background_tasks,
             http_response=sub_response,
+            json_rpc_request_class=self.request_class,
         ) as ctx:
             await ctx.enter_middlewares(self.entrypoint.middlewares)
 
@@ -850,13 +865,14 @@ class EntrypointRoute(APIRoute):
         errors: Sequence[Type[BaseError]] = None,
         common_dependencies: Sequence[Depends] = None,
         response_class: Type[Response] = JSONResponse,
+        request_class: Type[JsonRpcRequest] = JsonRpcRequest,
         **kwargs,
     ):
         name = name or 'entrypoint'
 
         _, path_format, _ = compile_path(path)
 
-        _Request = JsonRpcRequest
+        _Request = request_class
 
         common_dependant = Dependant(path=path_format)
         if common_dependencies:
@@ -914,6 +930,8 @@ class EntrypointRoute(APIRoute):
         self.app = request_response(self.handle_http_request)
         self.entrypoint = entrypoint
         self.common_dependencies = common_dependencies
+        self.request_class = request_class
+
 
     async def solve_shared_dependencies(
         self,
@@ -965,8 +983,7 @@ class EntrypointRoute(APIRoute):
         try:
             body = await self.parse_body(http_request)
         except Exception as exc:
-            logger.exception(str(exc), exc_info=exc)
-            resp = await self.entrypoint.handle_exception_to_resp(exc)
+            resp, _ = await self.entrypoint.handle_exception_to_resp(exc)
             response = self.response_class(content=resp, background=background_tasks)
         else:
             try:
@@ -1073,6 +1090,7 @@ class EntrypointRoute(APIRoute):
             http_request=http_request,
             background_tasks=background_tasks,
             http_response=sub_response,
+            json_rpc_request_class=self.request_class
         ) as ctx:
             await ctx.enter_middlewares(self.entrypoint.middlewares)
 
@@ -1129,6 +1147,7 @@ class Entrypoint(APIRouter):
         middlewares: Sequence[JsonRpcMiddleware] = None,
         scheduler_factory: Callable[..., Awaitable[aiojobs.Scheduler]] = aiojobs.create_scheduler,
         scheduler_kwargs: dict = None,
+        request_class: Type[JsonRpcRequest] = JsonRpcRequest,
         **kwargs,
     ) -> None:
         super().__init__(redirect_slashes=False)
@@ -1137,6 +1156,7 @@ class Entrypoint(APIRouter):
         self.middlewares = middlewares or []
         self.scheduler_factory = scheduler_factory
         self.scheduler_kwargs = scheduler_kwargs
+        self.request_class = request_class
         self.scheduler = None
         self.callee_module = inspect.getmodule(inspect.stack()[1][0]).__name__
         self.entrypoint_route = self.entrypoint_route_class(
@@ -1146,6 +1166,7 @@ class Entrypoint(APIRouter):
             errors=errors,
             dependencies=dependencies,
             common_dependencies=common_dependencies,
+            request_class=request_class,
             **kwargs,
         )
         self.routes.append(self.entrypoint_route)
@@ -1167,16 +1188,20 @@ class Entrypoint(APIRouter):
     async def handle_exception(self, exc):
         raise exc
 
-    async def handle_exception_to_resp(self, exc) -> dict:
+    async def handle_exception_to_resp(self, exc, log_unhandled_exception=True) -> Tuple[dict, bool]:
+        is_unhandled_exception = False
         try:
             resp = await self.handle_exception(exc)
         except BaseError as error:
             resp = error.get_resp()
         except HTTPException:
             raise
-        except Exception:  # noqa
+        except Exception as exc:
+            if log_unhandled_exception:
+                logger.exception(str(exc), exc_info=exc)
             resp = InternalError().get_resp()
-        return resp
+            is_unhandled_exception = True
+        return resp, is_unhandled_exception
 
     def bind_dependency_overrides_provider(self, value):
         for route in self.routes:
@@ -1207,6 +1232,7 @@ class Entrypoint(APIRouter):
             self.entrypoint_route.path + '/' + name,
             func,
             name=name,
+            request_class=self.request_class,
             **kwargs,
         )
         self.routes.append(route)
